@@ -30,7 +30,7 @@ impl HexOffset {
 
 impl Coordinate for HexOffset {}
 /// Defines wheter the hex graph is `flat-top` or `point-top` and is odd or even.
-pub trait HexOffsetShapeBase {
+pub trait HexOffsetShapeBase: Clone {
     type Axis: Axis;
     fn move_coord(
         horizontal: usize,
@@ -302,6 +302,7 @@ where
     H: Into<usize> + Copy,
     V: Into<usize> + Copy,
 {
+    /// Create a new shape.
     pub fn new(h: H, v: V) -> Self {
         Self {
             h,
